/* planck.c

Functions and constants to handle Planck's function and related functions

    Copyright (C) 1999, 2000 Conrado Badenas <Conrado.Badenas@uv.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "planck.h"

#ifdef HAVE_FENV_H
#include <fenv.h>
#endif HAVE_FENV_H

#include <math.h>
#include "number.h"
#include "macros.h"

static Number cut1=0.,cut2=0.,cut3=0.; /* Cut points for functions */
static Number lMAX=0.; /* Natural logarithm of MAXIMUM */
#define TESTCUT1 if (cut1<=0.) cut1=sqrt(EPSILON/sqrt(3.));
#define TESTCUT2 if (cut2<=0.) cut2=pow(240.*EPSILON/sqrt(3.),0.2);
#define TESTCUT3 if (cut3<=0.) cut3=pow(EPSILON*sqrt(3.),1./3.);
#define TESTLMAX if (lMAX<=0.) lMAX=log(MAXIMUM);

static Number C1=0.,erC1=0.,C2=0.,erC2=0.; /* Radiative "constants" */
static int Hold=0;

void i_planck(Number c1,Number erc1,Number c2,Number erc2,int hold)
{
	if (Hold) FATAL("i_planck(): Radiative constants cannot be changed\n")
	if (c1>0. && erc1>0. && c2>0. && erc2>0.)
		{C1=c1;erC1=erc1;C2=c2;erC2=erc2;Hold=hold;}
	else if (c1<=0. && erc1<=0. && c2<=0. && erc2<=0.) {Hold=hold;
		C1=1.191042723e-5;  /* mW/(m2 sr cm-4) er=7.8e-8 */
		erC1=7.8e-8;        /* Relative uncertainty of C1 */
		C2=1.4387752;       /* K/cm-1          er=1.7e-6 */
		erC2=1.7e-6;        /* Relative uncertainty of C2 */
/* These constants have been determined with C1=2hc**2 and C2=hc/k
h = 6.62606876(52)e-34 J s, er=7.8e-8
c = 299792458 m s-1, er=0 (exact figure)
k = 1.3806503(24)e-23 J K-1, er=1.7e-6
corr(h,k)=0.035; corr(h,c)=0.000; corr(k,c)=0.000
er is relative error; corr is correlation between physical constants */
	}
	else FATAL("i_planck(): Mixture of valid and non-valid arguments\n")
#ifdef HAVE_FENV_H
	if (fesetround(FE_TONEAREST)) WARN("i_planck(): Rounding mode"
		" FE_TONEAREST not supported by math hardware (FPU)\n")
#else
	WARN("i_planck(): Your system does not include the header file `fen"
		"v.h', and the program cannot set the rounding mode of your"
		" math hardware (FPU). In order to enable this feature, and"
		" disable this message, upgrade your system with a complete"
		" ISO C environment, configure, and compile the program.\n")
#endif
}

void g_planck(Number *c1,Number *erc1,Number *c2,Number *erc2,int *hold)
{
	*c1=C1;*erc1=erC1;*c2=C2;*erc2=erC2,*hold=Hold;
}

Number B(Number nu,Number t)
{
	Number b,x,y;
	
	if (t  <0.) FATAL2("B(): Invalid temperature %g\n",t)
	if (nu <0.) FATAL2("B(): Invalid wavenumber %g\n",nu)
	if (t ==0. || nu==0.) return 0.;
	if (C1<=0. || C2<=0.) i_planck(0.,0.,0.,0.,0);
	x=C2*nu/t; if (x<MINIMUM)  b=C1*nu*t*nu/C2;
	else {TESTLMAX if (x>lMAX) b=C1*nu*nu*nu*exp(-x);
		else {
#ifdef HAVE_EXPM1
			y=expm1(x);
#else
			TESTCUT1 if (x<cut1) y=x; else y=exp(x)-1.;
#endif
			b=C1*nu*nu*nu/y;
		}
	}
	if (b<MINIMUM) WARN4("B(): b=%g is denormalized"
		" for nu=%g, t=%g\n",b,nu,t)
	return b;
	
}

Number T(Number b,Number nu)
{
	Number t,x,y;
	
	if (b  <0.) FATAL2("T(): Invalid radiance %g\n",b)
	if (nu <0.) FATAL2("T(): Invalid wavenumber %g\n",nu)
	if (b ==0.) return 0.;
	if (nu==0.) FATAL2("T(): Radiance (%g) is impossible for nu=0\n",b)
	if (C1<=0. || C2<=0.) i_planck(0.,0.,0.,0.,0);
	y=C1*nu*nu*nu/b; if (y<MINIMUM) t=C2*b/C1/nu/nu;
	else {
#ifdef HAVE_LOG1P
		x=log1p(y);
#else
		TESTCUT1 if (y<cut1) x=y; else x=log(1.+y); 
#endif
		t=C2*nu/x;
	}
	if (t<MINIMUM) WARN4("T(): t=%g is denormalized"
		" for b=%g, nu=%g\n",t,b,nu)
	return t;
}

Number dBdT(Number nu,Number t)
{
	Number dbdt,x,z,w,ex;

	if (t  <0.) FATAL2("dBdT(): Invalid temperature %g\n",t)
	if (nu <0.) FATAL2("dBdT(): Invalid wavenumber %g\n",nu)
	if (t ==0. || nu==0.) return 0.;
	if (C1<=0. || C2<=0.) i_planck(0.,0.,0.,0.,0);
	x=C2*nu/t; if (x<MINIMUM)  dbdt=C1*nu*nu/C2;
	else {TESTLMAX if (x>lMAX) dbdt=C1*C2*nu*nu*nu*nu*exp(-x)/t/t;
		else {
#ifdef HAVE_EXPM1
			w=x/expm1(x);z=w*exp(x)*w;
#else
			TESTCUT2 if (x<cut2) z=1.-x*x/12.;
				 else {ex=exp(x);w=x/(ex-1.);z=w*ex*w;}
#endif
			dbdt=C1*nu*nu*z/C2;
		}
	}
	if (dbdt<MINIMUM) WARN4("dBdT(): dbdt=%g is denormalized"
		" for nu=%g, t=%g\n",dbdt,nu,t)
	return dbdt;
} 

Number alpha(Number x)
{
	if (x<0.) FATAL2("alpha(): Can't process x=%g\n",x)
	TESTCUT3 if (x<cut3) return 1.+0.5*x;
	         else        return x/(1.-exp(-x));
}
