/* srf2radi.c

Program to compute look-up tables of temperature and band integrated radiance
related data for any sensor with a known sensor response function (SRF)

Input:  sensor response function
Output: T (K), B(T) (mW m-2 sr-1), dB/dT (mW m-2 sr-1 K-1),
   num_nueffnum, [nueffnumi (cm-1), deltarTnum_i (K)] (i=1,num_nueffnum),
   num_nueffanl, [nueffanli (cm-1), deltarTanl_i (K)] (i=1,num_nueffnum).

    Copyright (C) 1999, 2000, 2001 Conrado Badenas <Conrado.Badenas@uv.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <math.h>

#include "number.h"
#include "macros.h"
#include "planck.h"
#include "dvector.h"
#include "snsrspfn.h"
#include "search.h"

Number ident(Number nu,Number t) {return 1.;}
Number alphaeq3(Number x) {return alpha(x)-3.;}

static Number t,b;
Number fnueq0(Number nu) {return B(nu,t)-b;}

int main(int argc, char *argv[])
{
	int argverb,verbose=0,ngaps,i;
	Number t0,tn,x0,c1,erc1,c2,erc2,er=EPSILON,invavgident;
	dVector nu,srf;

	if (argc==1) {printf("Version: " VERSION "\nUsage: srf2radi t"
		"emp_0 [temp_n num_gaps] { [--verbose] | [--Verbose] "
		"} [<srf_file] [>radi_file]\n");exit(1);}
	if ((t0=atof(argv[1]))<0.)
		FATAL2("temp_0=%g not allowed\n",t0)
	if (argc<=3) {ngaps=0;tn=t0;argverb=2;}	else {
		argverb=4;
		if ((tn=atof(argv[2]))<0.)
			FATAL2("temp_n=%g not allowed\n",tn)
		if ((ngaps=atoi(argv[3]))<0 || (!ngaps && tn!=t0))
			FATAL2("num_gaps=%d not allowed\n",ngaps)
	}
	if (argc>argverb) {
		if (!strcmp(argv[argverb],"--verbose")) verbose=1;
		else if (!strcmp(argv[argverb],"--Verbose")) verbose=2;
		else FATAL2("`%s' is neither `--verbose' nor `--Verbos"
			"e'\n",argv[argverb])
	}
	if (t0>tn || (t0==tn && ngaps!=0))
		FATAL3("temp_0 (%g) >= temp_n (%g)\n",t0,tn)
	load_srf(stdin,&nu,&srf);g_planck(&c1,&erc1,&c2,&erc2,&i);
	x0=rootfunc(alphaeq3,2.,3.,0.,EPSILON,0);
	invavgident=1./intgrt_srf(ident,0.,&nu,&srf);
	for (i=0;i<=ngaps;i++)
	{
		unsigned solsnum,sols,sol,solsanl;
		Number nuM,dbdt,numin,numax,nueffnum[2];
		Number xm,a1,a2,a3,a4,x[3],nueffanl[3];

		if (ngaps) t=t0+i*(tn-t0)/ngaps; else t=t0;
		b   =invavgident*intgrt_srf( B  ,t,&nu,&srf);
		dbdt=invavgident*intgrt_srf(dBdT,t,&nu,&srf);
		printf(PND TAB PND TAB PND,t,b,dbdt);
		if (verbose)
		{
			nuM=x0*t/c2;solsnum=0;
			numin=g_dvector(&nu,0);numax=g_dvector(&nu,nu.n-1);
			if (nuM<numin||nuM>numax) nueffnum[solsnum++]=
				rootfunc(fnueq0,numin,numax,0.,er,0);
			else
			{
				if (B(numin,t)<=b) nueffnum[solsnum++]=
					rootfunc(fnueq0,numin,nuM  ,0.,er,0);
				if (B(numax,t)<=b) nueffnum[solsnum++]=
					rootfunc(fnueq0,nuM  ,numax,0.,er,0);
			}
			printf("\t%u",solsnum);for (sol=0;sol<solsnum;sol++) {
				printf(TAB PND,nueffnum[sol]);
				if (t!=0.) printf(TAB PND,
					T(b,nueffnum[sol])/t-1);
				else printf("\t*");
			}
		if (verbose==2) {
			solsanl=0;xm=0.5*c2*(numin+numax)/t;
			a1=c1/(exp(xm)*b);a2= -0.5*c2*c2/(t*t);
			a3=c2*(xm-1.)/t;a4=exp(-xm)-1.+xm-0.5*xm*xm;
			sols=rootcub(a1,a2,a3,a4,&x[0],&x[1],&x[2]);
			for (sol=0;sol<sols;sol++)
				if ((xm=x[sol])>=numin && xm<=numax)
					nueffanl[solsanl++]=xm;
			printf("\t%u",solsanl);for (sol=0;sol<solsanl;sol++) {
				printf(TAB PND,nueffanl[sol]);
				if (t!=0.) printf(TAB PND,
					T(b,nueffanl[sol])/t-1);
				else printf("\t*");
			}
		} }
		printf(NWL);
	}
/*	printf (PND TAB PND TAB PND NWL,MAXIMUM,MINIMUM,EPSILON); */
	return 0;
}
